/**************************************************************
 * core object system
 *
 * written by Doug Lanford, opus@opusgames.com
 * for more info, go to www.opusgames.com
 *
 * This file is provided as freeware
 *************************************************************/

#include "object.h"
#include "explosion.h"
#include "ship.h"
#include "bullet.h"
#include "powerup.h"

// buffer of game objects
ObjectStruct objectBuffer[MAX_OBJECTS];

// buffer of active objects (sorted by object type)
u16 activeObjects[NUM_OBJECT_TYPES];

// buffer of unused objects
u16 unusedObjects = 0;

// each new object is given a unique nID
u16 next_nID = 1;

// list of update functions for each type of object
typedef u32 (*ObjectUpdateFunction)	(ObjectStruct *pObj);
ObjectUpdateFunction objectUpdateFunctions[NUM_OBJECT_TYPES] =
{
	Explosion_Update,	// update routine for explosions
	Ship_Update,		// update routine for computer controlled ships
	Bullet_Update,		// update routine for bullets
	Powerup_Update,		// update routine for powerups
};


/************************************************************************
 * get the pointer to an object from its handle
 *
 * IN:		32 bit handle to the object
 *
 * RETURNS:	ptr to the handle's ObjectStruct, NULL if object has died
 ************************************************************************/
ObjectStruct *Object_getPtr (u32 id)
{
	// grab the nID value from the id (top word of the id)
	u16 nID = *(((u16 *)&id) + 1);
	// grab the ptr to the object (from index, lower word of id)
	ObjectStruct *pObj = &objectBuffer[id & 0x0000FFFF];

	// is object still valid?
	if (nID != pObj->nID)
	{	// no... object has died and it's buffer slot has been reallocated
		return(NULL);
	}

	// object is still valid... return its pointer
	return(pObj);
}


/************************************************************************
 * initialize the object system
 ***********************************************************************/
void Object_InitSystem (void)
{
int i;

	for (i=0; i<MAX_OBJECTS; i++)
	{	// init and link up all the unused objects
		objectBuffer[i].nID = 0;
		objectBuffer[i].next = i+1;
	}
	objectBuffer[i].next = OBJECT_NULL;	// terminate linked list

	unusedObjects = 0;	// top of unused list is object 0

	for (i=0; i<NUM_OBJECT_TYPES; i++)
	{	// init active list to empty
		activeObjects[i] = OBJECT_NULL;
	}
}


/************************************************************************
 * get a new game object
 *
 * IN:		type of object
 *
 * RETURNS:	handle to the object
 *
 * NOTE:	the handle for an object is a 32 bit value:
 * 				lower 16 bits = object number in object buffer
 * 				upper 16 bits = object's nID (useful for tracking if the object has died)
 ************************************************************************/
u32 Object_New (u16 type)
{
u16 newObjIdx;
ObjectStruct *pObj;

	newObjIdx = unusedObjects;
	if (newObjIdx == OBJECT_NULL)
	{	// no objects left
		return(OBJECT_ID_NULL);
	}

	pObj = &objectBuffer[newObjIdx];

	// get next unused object
	unusedObjects = pObj->next;

	// add new object to the top of it's active list
	pObj->next = activeObjects[type];
	pObj->prev = OBJECT_NULL;
	objectBuffer[pObj->next].prev = newObjIdx;
	activeObjects[type] = newObjIdx;

	// init object
	pObj->type = type;
	pObj->nID = next_nID++;
	if (next_nID == 0)
	{	// zero is invalid nID (for unused objects)
		next_nID++;
	}

	// build the object handle (combined nID and object index in buffer)
	return((pObj->nID<<16) + newObjIdx);
}


/******************************************************************************
 * deletes an object from the system (returns it to the unused buffer)
 *
 * IN:		object handle
 *****************************************************************************/
void Object_Kill (u32 id)
{
ObjectStruct *pObj;

	pObj = Object_getPtr(id);
	if (pObj == NULL)
	{	// object is already dead
		return;
	}

	// remove it from its active list
	if (pObj->prev == OBJECT_NULL)
	{	// this object was at the top of its list... move next one up to top
		activeObjects[pObj->type] = pObj->next;
		objectBuffer[activeObjects[pObj->type]].prev = OBJECT_NULL;
	}
	else
	{	// object was in the middle of the list
		objectBuffer[pObj->prev].next = pObj->next;

		if (pObj->next != OBJECT_NULL)
		{	// object was not end of list
			objectBuffer[pObj->next].prev = pObj->prev;
		}
	}

	// add the object to the unused list
	pObj->nID = OBJECT_ID_NULL;
	pObj->next = unusedObjects;

	unusedObjects = id & 0x0FFFF;
}



/******************************************************************************
 * update all the active objects
 *****************************************************************************/
void Object_UpdateAll (void)
{
u32 type;
u32 curIdx;

	// walk through entire active buffer
	for (type=0; type<NUM_OBJECT_TYPES; type++)
	{
		curIdx = activeObjects[type];
		while (curIdx != OBJECT_NULL)
		{	// there are still object left in the list
			// update it
			if ((objectUpdateFunctions[type])(&objectBuffer[curIdx]))
			{	// object returned that it is now dead
				u32 nextIdx = objectBuffer[curIdx].next;	// grab the next object to update

				// kill this object
				Object_Kill((objectBuffer[curIdx].nID<<16) + curIdx);

				curIdx = nextIdx;
			}
			else
			{	// goto next object
				curIdx = objectBuffer[curIdx].next;
			}
		}
	}
}




